/**
 * @fileOverview
 * @name kth.js
 * @author Kim il (blaxtar@kthcorp.com)
 * @version 20120625
 * @lastModified 2012-06-25
 * @created 2011-12-21 
 * @description
 * <pre>
 * KTH 라이브러리 패키지, KTH Libraries 및 Top Level 함수 모음
 *
 * functions:
 * - {@link kth.getPackage} 네임스페이스 스트링을 이용해 패키지로 사용할 빈 오브젝트 혹은 기존의 오브젝트를 반환
 * - {@link kth.replaceStr} 템플릿 문자열을 JSON 형태의 데이터 값으로 매핑하여 치환
 * - {@link kth.getRandom} 주어진 범위에서 임의의 정수를 추출
 * - {@link kth.getOffsetIndex} 임의의 갯수의 배열중 기준이 될 인덱스로부터 지정된 옵셋 위치의 인덱스를 반환
 * </pre>
 */

/**
 * KTH 라이브러리 패키지
 * @namespace KTH 라이브러리 패키지
 * @name kth
 */
kth = window.kth || {};

/**
 * 네임스페이스 스트링을 이용해 패키지로 사용할 빈 오브젝트 혹은 기존의 오브젝트를 반환
 * @author Kim il (blaxtar@kthcorp.com)
 * @param {string} packageName 네임스페이스로 사용할 이름
 * @param {object} object 네임스페이스에 선언될 오브젝트 본체
 * @param {boolean} keepPrevious 해당 네임스페이스가 이미 존재할 경우 기존의 오브젝트를 override 하지 않는다
 * @return {object} 해당 네임스페이스에 선언된 오브젝트
 * @example
// usage 1
var utils = kth.getPackage("kth.utils");
console.log(utils); // Object {}

// usage 2
kth.getPackage("kth.utils").imageViewer = function(){
	// code...
}
console.log(kth.utils.imageViewer); // function()

// usage 3
(function($namespace){
	$namespace["cutString"] = function(){
		// code...
	};
	console.log(kth.utils.cutString); // function()
	$namespace["clone"] = function(){
		// code...
	};
	console.log(kth.utils.clone); // function()
})( kth.getPackage("kth.utils") );

// usage 4 :: Specify object in namespace 
kth.getPackage("kth.utils.numberUtils",{
	getRandomInt: function(){
		// code...
	},
	getRactanglePoint: function(){
		// code...
	}
});
console.log(kth.utils.numberUtils); // Object { specified... }

// 이미 존재하는 namespace이지만 세번째 parameter가 지정되지 않았으므로 override한다
kth.getPackage("kth.utils.numberUtils", {
	newObject: "overriden!"
});
console.log(kth.utils.numberUtils); // Object { newObject="overriden!" } 

// 이미 존재하는 namespace이며 세번째 parameter가 true이므로 새로운 object로 override 하지 않음
kth.getPackage("kth.utils.numberUtils", {
	newObject: "not overriden"
}, true);
console.log(kth.utils.numberUtils); // Object { specified... }
	
// usage 5 :: Specify object wrapped in closure
kth.getPackage("kth.utils.stringUtils", (function(){
	// internal constants
	var const1 = "...";
	var const2 = 123123;
	
	// object body
	return {
		getBytes: function()
		{
			// code...
		},
		trim: function()
		{
			// code....
		}
	}
})());
console.log(kth.utils.stringUtils); // Object { specified... }
 **/
kth.getPackage = function(packageName, object, keepPrevious){
	var parts = packageName.split(".");
	var part = window;
	
	for (var i = 0, length = parts.length, lastIndex = parts.length-1; i < length; i++){
		part = (part[parts[i]] === undefined) ?
			( part[parts[i]] = (i == lastIndex && object) ? object : {} ) :
			( (i == lastIndex && object && !keepPrevious) ? (part[parts[i]] = object) : part[parts[i]] );
	}
	
	return part;
};

/**
 * 템플릿 문자열을 JSON 형태의 데이터 값으로 매핑하여 치환
 * @author Kim il (blaxtar@kthcorp.com)
 * @param {string} template 템플릿 문자열. 치환될 문자열은 ${var}의 형태로 기술  
 * @param {object} data JSON 데이터 값
 * @return {string} 치환된 문자열
 * @example
var replacedStr = kth.replaceStr("안녕하세요. 저는 ${name}이고, 나이는 ${age}입니다.", {
	name: "김일",
	age: 31
});
console.log(replacedStr) // "안녕하세요. 저는 김일이고, 나이는 31입니다."
 **/
kth.replaceStr = function(template, data){
	var reg = (arguments.length == 2) ? /\${([A-Za-z0-9_]+)}/g : arguments[2];
	return template.replace(reg, function(m, p) {
		return (data[p] !== undefined) ? data[p] : "";
	});
};

/**
 * 주어진 범위에서 임의의 정수를 추출
 * @author Kim il (blaxtar@kthcorp.com)
 * @param {number} min 최소값
 * @param {number} max 최대값
 * @return {number} 임의의 정수
 * @example
var randomizedNumber = kth.getRandom(1, 10);
console.log(randomizedNumber) // 1~10
 */
kth.getRandom = function(min, max){
	return Math.floor(Math.random() * (max - min + 1)) + min;
};


/**
 * 임의의 갯수의 배열중 기준이 될 인덱스로부터 지정된 옵셋 위치의 인덱스를 반환
 * @param {number} index 기준이 될 인덱스
 * @param {number} offset 기준이 될 인덱스로부터의 옵셋
 * @param {number} length 배열의 전체 갯수
 * @param {boolean} infinite 배열의 원소가 무한대로 이어지는지의 여부
 * @return {number} 배열의 인덱스
 * @example
var array = [0, 1, 2, 3, 4, 5],
	offsetIndex = getOffsetIndex(0, -2, array.length);
console.log(offsetIndex, array[offsetIndex]); // 4, 4
 */
kth.getOffsetIndex = function(index, offset, length, infinite){
	var i = (arguments.length >= 4) ? !!infinite : true,
		a = index + offset, l = length;
	
	return (!i && ((a < 0) || (a > (l - 1)))) ? -1 : (a - l * Math.floor(a / l));
};

/**
 * URL의 쿼리스트링을 파싱하여 주어진 키와 매칭되는 값, 혹은 전체 값이 저장된 객체를 반환
 * @param {number} [queryKey] 찾을 키
 * @return {string|object} 키와 매칭되는 값 혹은 쿼리스트링으로부터 파싱된 객체
 */
kth.getQueryVariable = function getQueryVariable(queryKey) {
	var query, vars, pair, key, value, queryVariables;
	getQueryVariable = arguments.callee || getQueryVariable;
	
	if ((queryVariables = getQueryVariable.queryVariables)) {
		return queryKey ? queryVariables[queryKey] : queryVariables;
	}
	
	query = window.location.search.substring(1);
	vars = query.split("&");
	queryVariables = {};
	
	for (var i = 0; i < vars.length; i++) {
		pair = vars[i].split("=");
		if (pair.length < 2 || !(key = pair[0])) {
			continue;
		} else if (!(value = pair[1])) {
			value = '';
		}
		queryVariables[key] = decodeURIComponent(value);
	}
	getQueryVariable.queryVariables = queryVariables;
	return queryKey ? queryVariables[queryKey] : queryVariables;
};