/**
 * @fileOverview
 * @name kth.puddingto.signUp.js
 * @author Kim il (blaxtar@kthcorp.com)
 * @version 20121119
 * @lastModified 2012-11-19
 * @created 2012-06-18
 * @description
 * <pre>
 * Pudding.to > 가입하기 페이지
 *
 * classes:
 * - {@link kth.puddingto.SignUp} '가입하기' 및 '약관/개인정보 취급방침' 통합 페이지 엔트리 포인트
 * - {@link kth.puddingto.SignUpForm} 가입하기 폼
 * - {@link kth.puddingto.SignUpInputField} 가입하기 입력 필드
 * </pre>
 */
/*jslint browser:true, devel:true, white:false, nomen: true, plusplus:true, sloppy:true */
(function(global) {

/**
 * 객체의 속성 갯수를 구함
 * @param {object} obj 객체
 * @return {numbers} 객체의 속성 갯수
 */
function size(obj) {
	var keys = [],
		key;
	for (key in obj) {
		keys[keys.length] = key;
	}
    return keys.length;
}

/**
 * @class
 * @name kth.puddingto.SignUpForm
 * @param {HTMLElement} element HTML Element
 * @example
kth.puddingto.SignUpForm(element);
 */
kth.getPackage('kth.puddingto.SignUpForm', (function(NAME, SignUpForm) {
	var /* private static constants */
		_INSTANCE = null;
	
	/**
	 * @constructor
	 * @ignore
	 */
	SignUpForm = function SignUpForm(element) {
		if (_INSTANCE) {
			return _INSTANCE;
		} else if (!(this instanceof SignUpForm)) {
			return (_INSTANCE = new SignUpForm(element));
		} else {
			_INSTANCE = this;
		}
		this._init(element);
	};
	SignUpForm.prototype = /** @lends kth.puddingto.SignUpForm.prototype */ {
		_validators: {
			'email': function(value) {
				if (!value) {
					return PuddingTo.i18n['FILL_FIELDS'];
				} else if (!value.match(PuddingTo.consts.REGEX_EMAIL)) {
					return PuddingTo.i18n['EMAIL_INVALID'];
				} else {
					return this._checkDuplication('email', value, this._onDuplicationCheckSuccess); 
				}
			},
			
			'username': function(value) {
				if (!value) {
					return PuddingTo.i18n['FILL_FIELDS'];
				} else if (value.length > 15) {
					return PuddingTo.i18n['USERNAME_INVALID'];
				} else if (!value.match(PuddingTo.consts.REGEX_USERNAME)) {
					return PuddingTo.i18n['USERNAME_INVALID'];
				} else {
					return this._checkDuplication('username', value, this._onDuplicationCheckSuccess); 
				}
			},
			
			'password': function(value) {
				if (!value) {
					return PuddingTo.i18n['FILL_FIELDS'];
				} else if (value.length < 6 || value.length > 64) {
					return PuddingTo.i18n['PASSWORD_INVALID'];
				} else if (!value.match(PuddingTo.consts.REGEX_PASSWORD)) {
					return PuddingTo.i18n['PASSWORD_INVALID'];
				}
				if (this._getValue('password_confirmation')) {
					this._validate('password_confirmation');	
				}
			},
			
			'password_confirmation': function(value) {
				if (!value) {
					return PuddingTo.i18n['FILL_FIELDS'];
				} else if (!value.length) {
					return PuddingTo.i18n['PASSWORD_CONFIRMATION_REQUIRED'];
				} else if (value !== this._getValue('password')) {
					return PuddingTo.i18n['PASSWORD_CONFIRMATION_INVALID'];
				}
			},
			
			'agreed_terms': function(value) {
				if (!value) {
					return PuddingTo.i18n['TERM_REQUIRED'];
				}	
			}
		},
		
		element: null,
		_backButton: null,
		_closeButton: null,
		_signUpButton: null,
		_form: null,
		
		_inputFields: null,
		_inputFieldsNames: null,
		_values: null,
		_isSigningUp: false,
		
		_errors: null,
		_remoteValidators: null,
		_remoteValidatorCallbacks: null,
		
		_init: function(element) {
			this.element = $(element);
			
			this._onInputFieldChange = this._onInputFieldChange.bind(this);
			this._onDuplicationCheckSuccess = this._onDuplicationCheckSuccess.bind(this);
			this._onSignUp = this._onSignUp.bind(this);
			this._onRequestSignUpError = this._onRequestSignUpError.bind(this);
			
			this._backButton = $('A[data-call=back]').click(this._onBackButtonClick.bind(this));
			this._closeButton = $('A[data-call=close]').click(this._onCloseButtonClick.bind(this));
			this._signUpButton = $('A[data-call=signup]').click(this._onSignUp);
			this._form = $('FORM[data-call=signup]').bind('submit', this._onSignUp);
			
			this._errors = {};
			this._remoteValidators = {};
			this._remoteValidatorCallbacks = {};
			
			this._initInputFields();
			this._initRegisterId();
		},
		
		_initInputFields: function() {
			this._inputFieldsNames = [];
			this._inputFields = $.map([
				$('[name=email]'),
				$('[name=username]'),
				$('[name=password]'),
				$('[name=password_confirmation]'),
				$('[name=agreed_terms]')
			], function(element) {					
				var signUpInputField = element.length ? new kth.puddingto.SignUpInputField(element, {
						onChange: this._onInputFieldChange
					}) : undefined,
					name;
				
				if (signUpInputField) {
					name = signUpInputField.name;
					signUpInputField.setValue(this._setValue(name, PuddingTo.params[name]));
					this._inputFieldsNames.push(name);
				}
				return signUpInputField;
			}.bind(this));
		},
		
		_initRegisterId: function() {
			if (!PuddingTo.params.register_id) {
				PuddingTo.API.call('get register', {
					success: function(data) {
						PuddingTo.params.register_id = data;
					}
				});
			}
		},
		
		_onBackButtonClick: function(event) {
			event.preventDefault();
			this._backButton.addClass('on');
			history.back();
		},
		
		_onCloseButtonClick: function(event) {
			event.preventDefault();
			this._closeButton.addClass('on');
			location.href = 'puddingsns://CLOSE';
			console.log('puddingsns://CLOSE');
		},
		
		_onInputFieldChange: function(event, inputField) {
			this._validate(inputField.name, inputField.getValue());
		},
		
		_validate: function(key, value) {
			var enforce = arguments.length < 2,
				prevValue = this._getValue(key),
				hasChanged = this._setValue(key, (enforce ? (value = prevValue) : value)) !== prevValue,
				validator = this._validators[key],
				validationResult = null,
				error = null,
				remoteValidator = null,
				inputField = this._getInputField(key);
			
			if (!enforce && !hasChanged) {
				return this._remoteValidators[key];
			}
			
			validationResult = validator.apply(this, [value]);
			
			if (validationResult) {
				if ($.isFunction(validationResult)) {
					remoteValidator = this._remoteValidators[key] = validationResult;
				} else {
					error = validationResult;
				}
			}
			if (error) {
				this._addError(key, error);
				if (inputField) {
					inputField.setIndicator(error);	
				}
			} else if (remoteValidator) {
				this._removeError(key);
				if (inputField) {
					inputField.hideIndicator();	
				}
			} else {
				this._removeError(key);
				if (inputField) {
					inputField.setIndicator();	
				}
			}
			
			if (remoteValidator || error) {
				return remoteValidator || error;
			}
		},
		
		_validateAll: function(callback) {
			var validationResult,
				hasRemoteValidator = false;
			
			$.each(this._values, function(key) {
				if (this._getError(key)) {
					return;
				}
				validationResult = this._validate(key);
				
				if (validationResult && $.isFunction(validationResult)) {
					hasRemoteValidator = true;
					this._remoteValidatorCallbacks[key] = function(key) {
						this._remoteValidatorCallbacks[key] = null;
						delete this._remoteValidatorCallbacks[key];
						
						if (!size(this._remoteValidatorCallbacks)) {
							this._onValidateAllComplete(callback);
						}
					}.bind(this);
				}
			}.bind(this));
			
			if (!hasRemoteValidator) {
				this._onValidateAllComplete(callback);
			}
		},
		
		_onValidateAllComplete: function(callback) {
			if (size(this._errors) && $.isFunction(callback)) {
				callback(this._errors);
			} else if ($.isFunction(callback)) {
				callback();
			}
		},
		
		_checkDuplication: function(key, value, callback) {
			var params = {};
				
			params[key] = value;
			
			PuddingTo.API.call('get user/duplicate', {
				data   : params,
				global : false,
				success: function(data) {
					callback.apply(this, [key, value, data === 'yes']);
				},
				complete: function() {
					var callback = this._remoteValidatorCallbacks[key];
					
					this._remoteValidators[key] = null;
					delete this._remoteValidators[key];
					
					if ($.isFunction(callback)) {
						callback(key);
					}
				}.bind(this)
			});
			
			return callback;
		},
		
		_onDuplicationCheckSuccess: function(key, value, isDuplicated) {
			var error = PuddingTo.i18n[key.toUpperCase() + '_DUPLICATED'];
			
			this._setValue(key, value);
			this._getInputField(key).setIndicator(isDuplicated ? error : "");
			
			if (isDuplicated) {
				this._addError(key, error);
			} else {
				this._removeError(key);
			}
		},
		
		_setValue: function(key, value) {
			return ((this._values || (this._values = {}))[key] = value);
		},
		
		_getValue: function(key) {
			return this._values[key];
		},
		
		_getInputField: function(key) {
			return this._inputFields[this._inputFieldsNames.indexOf(key)];
		},
		
		_onSignUp: function(event) {
			var isSignUpOnFacebook = (this._signUpButton.attr('data-options') === 'fb');
			
			event.preventDefault();
			
			if (this._isSigningUp) {
				return;
			} else {
				this._isSigningUp = true;	
			}
			
			this._signUpButton.addClass('on');
			
			this._validateAll(function(errors) {
				var error,
					firstErrorOccurredInputField;
					
				if (errors) {
					$.each(this._inputFields, function(index, inputField) {
						var name = inputField.name;
						if (error) {
							return;
						}
						
						if (errors[name]) {
							error = errors[name];
							firstErrorOccurredInputField = inputField;
						}
					}.bind(this));
					this._onSignUpAborted(error, firstErrorOccurredInputField);
				} else {
					setTimeout(function() {
						if (!confirm(Mustache.to_html(PuddingTo.i18n.EMAIL_CONFIRM, {
							email: this._getValue('email')
						}))) {
							return this._onSignUpAborted();
						}
						this._requestSignUp(isSignUpOnFacebook);	
					}.bind(this), 100);
				}
			}.bind(this));
		},
		
		_onSignUpAborted: function(errorMessage, inputField) {
			if (errorMessage) {
				setTimeout(function() {
					alert(errorMessage);	
				}, 100);
			}
			if (inputField) {
				inputField.setFocus();				
			}
			this._isSigningUp = false;
			this._signUpButton.removeClass('on');
		},
		
		_requestSignUp: function(isSignUpOnFacebook) {
			var data = {
					email : this._getValue('email'),
					username : this._getValue('username'),
					password : this._getValue('password'),
					client_id : PuddingTo.params.client_id,
					register_id: PuddingTo.params.register_id,
				};
	
			if (isSignUpOnFacebook) {
				data.register_id = PuddingTo.params.register_id;
				data.fb = PuddingTo.params.fb;
				data.uid = PuddingTo.params.uid;
			}
			
			PuddingTo.API.call('post register', {
				data : data,
				success: function(data) {
					location.href = PuddingTo.urlFor(PuddingTo.consts[$.os.android ? 'CALL_LOGIN_OK' : 'URL_LOGIN_OK'], data);
				},
				error: this._onRequestSignUpError
			});
		},
		
		_onRequestSignUpError: function(data) {
			this._onSignUpAborted();
		},
		
		_addError: function(key, error) {
			return (this._errors[key] = error);
		},
		
		_getError: function(key) {
			return this._errors[key];
		},
		
		_removeError: function(key) {
			this._errors[key] = null;
			delete this._errors[key];
		},
		
		hide: function() {
			this.element.hide();
			return this;
		},
		
		show: function() {
			this.element.show();
			return this;
		},
		
		setTermsAgreement: function(termsAgreement) {
			var key = 'agreed_terms';
			this._getInputField(key).setValue(termsAgreement);
			this._validate(key, termsAgreement);
		},
		
		__testSignUp__: function() {
			if (!this.test) {
				this.test = true;
				this._setValue('email', this._getInputField('email').setValue('test@test.com').getValue());
				this._setValue('username', this._getInputField('username').setValue('tasdfljasldkfj').getValue());
				this._setValue('password', this._getInputField('password').setValue('123123').getValue());
				this._setValue('password_confirmation', this._getInputField('password_confirmation').setValue('123123').getValue());
				this._setValue('agreed_terms', this._getInputField('agreed_terms').setValue(true).getValue());
			}
			
			this._onSignUp({
				preventDefault: function() {}
			});
		}
	};
	return SignUpForm;
})('kth.puddingto.SignUpForm'));

/**
 * @class
 * @name kth.puddingto.SignUpInputField
 * @param {HTMLElement} element Base Element
 * @example
kth.puddingto.SignUpInputField(element);
 */
kth.getPackage('kth.puddingto.SignUpInputField', (function(NAME, SignUpInputField) {
	/**
	 * @constructor
	 * @ignore
	 */
	SignUpInputField = function SignUpInputField(element, options) {
		var options = this.options = $.extend(this.options, options);
		
		this._onBlur = this._onBlur.bind(this);
		this._onChange = this._onChange.bind(this);
		this._onIndicatorClick = this._onIndicatorClick.bind(this);
		
		
		this.element = $(element);
		this.name = this.element.attr('name');
		this.type = this.element.attr('type');
		
		if (this.type === 'checkbox') {
			this.element.bind('change', this._onChange);
		} else {
			this.element
					.bind('blur', this._onBlur)
					.bind('change', this._onChange);
			this.indicator = this.element.next().click(this._onIndicatorClick);
		}
	};
	SignUpInputField.prototype = /** @lends kth.puddingto.SignUpInputField.prototype */ {
		options: {
			validatedCSSClass: 'imb',
			invalidatedCSSClass: 'avb',
			onChange: function(){}
		},
		
		element: null,
		indicator: null,
		name: '',
		type: '',
		
		_errorMessage: '',
		_isValidated: false,
		
		_onBlur: function(event) {
			this._onChange(event);
		},
		
		_onChange: function(event) {
			var value = this.getValue();
			this.setValue((typeof value === 'string') ? value.replace(/\s/g, '') : value);
			this.options.onChange.apply(null, [event, this]);
		},
		
		_onIndicatorClick: function(event) {
			if (!this._isValidated) {
				alert(this._errorMessage);
				this.setFocus();
			}
		},
		
		getValue: function() {
			var value;
			if (this.type === 'checkbox') {
				value = this.element.attr('checked');
			} else {
				value = this.element.val();
			}
			return value;
		},
		
		setValue: function(value) {
			if (this.type === 'checkbox') {
				this.element.get(0).checked = value ? 'checked' : null;
			} else {
				this.element.val(value || "");
			}
			return this;
		},
		
		setFocus: function() {
			this.element.focus();
		},
		
		setIndicator: function(errorMessage) {
			this._errorMessage = errorMessage || '';
			this._isValidated = !!!this._errorMessage;
			
			if (this.indicator) {
				if (this._isValidated) {
					this.indicator.removeClass(this.options.invalidatedCSSClass);
					this.indicator.addClass(this.options.validatedCSSClass);
				} else {
					this.indicator.removeClass(this.options.validatedCSSClass);
					this.indicator.addClass(this.options.invalidatedCSSClass);
				}	
			}
		},
		
		hideIndicator: function() {
			this._errorMessage = '';
			this._isValidated = false;
			
			if (this.indicator) {
				this.indicator.removeClass(this.options.invalidatedCSSClass);
				this.indicator.removeClass(this.options.validatedCSSClass);
			}
		},
		
		isValidated: function() {
			return this._isValidated;
		},
		
		getErrorMessage: function() {
			return this._errorMessage;
		}
	};
	return SignUpInputField;
})('kth.puddingto.SignUpInputField'));

/**
 * @class
 * @name kth.puddingto.SignUp
 * @param {object} options Options
 * @example
kth.puddingto.SignUp({
	signUpFormSelector: '#signUpFormSelector',
	termsServiceSelector: '#termsServiceSelector'
});
 */
kth.getPackage('kth.puddingto.SignUp', (function(NAME, SignUp) {
	var /* private static constants */
		_INSTANCE = null;
	
	/**
	 * @constructor
	 * @ignore
	 */
	SignUp = function SignUp(options) {
		if (_INSTANCE) {
			return _INSTANCE;
		} else if (!(this instanceof SignUp)) {
			return (_INSTANCE = new SignUp(options));
		} else {
			_INSTANCE = this;
		}
		this._init(options);
	};
	SignUp.prototype = /** @lends kth.puddingto.SignUp.prototype */ {
		options: {
			signUpFormSelector: '#container',
			termsServiceSelector: '#terms-service'
		},
		
		_signUpForm: null,
		_termsService: null,
		
		_init: function(options) {
			var options = this.options = $.extend(this.options, options);
			
			this._onHashChange = this._onHashChange.bind(this);
			this._onTermsServiceBackButtonClick = this._onTermsServiceBackButtonClick.bind(this);
			this._onTermsServiceAgreementButtonClick = this._onTermsServiceAgreementButtonClick.bind(this);
			
			this._signUpForm = kth.puddingto.SignUpForm($(this.options.signUpFormSelector));
			
			$(window).bind('hashchange', this._onHashChange);
		},
		
		_onHashChange: function(event) {
			var url = window.location.hash,
				isTermsServiceTriggered = !!~url.indexOf('#terms-service');
			
			if (isTermsServiceTriggered) {
				(this._termsService || (this._termsService = 
					kth.puddingto.TermsService(this.options.termsServiceSelector, {
						onBackButtonClick: this._onTermsServiceBackButtonClick,
						onAgreementButtonClick: this._onTermsServiceAgreementButtonClick
					}))
				).show();
				kth.puddingto.SignUpForm().hide();
			} else {
				kth.puddingto.SignUpForm().show();
				kth.puddingto.TermsService().hide();
			}
		},
		
		_onTermsServiceBackButtonClick: function(event) {
			window.history.back();
		},
		
		_onTermsServiceAgreementButtonClick: function(event) {
			this._signUpForm.setTermsAgreement(true);
			window.history.back();
		}
	};
	return SignUp;
})('kth.puddingto.SignUp'))({
	signUpFormSelector: '#container',
	termsServiceSelector: '#terms-service'
});

})(window);