/**
 * @fileOverview
 * @name kth.puddingto.termsService.js
 * @author Kim il (blaxtar@kthcorp.com)
 * @version 20120719
 * @lastModified 2012-07-19
 * @created 2012-06-04
 * @description
 * <pre>
 * Pudding.to > 가입하기 > 약관/개인정보 취급방침
 *
 * classes:
 * - {@link kth.puddingto.TermsService} 약관/개인정보 취급방침 페이지
 * 
 * </pre>
 */
/*jslint browser:true devel:true white:false plusplus:false */
/*global window:true $:true kth:true */
(function(global) {

/**
 * @class
 * @name kth.puddingto.TermsService
 * @param {HTMLElement} element HTML Element
 * @param {object} options Options 
 * @example
kth.puddingto.TermsService(element, {
	onBackButtonClick: function() {},
	onAgreementButtonClick: function() {}
});
 */
kth.getPackage('kth.puddingto.TermsService', (function(NAME, TermsService) {
	var /* external references */
		i18n = kth.puddingto.i18n(),
		/* private static constants */
		_INSTANCE = null;
	
	/**
	 * @constructor
	 * @ignore
	 */
	TermsService = function TermsService(element, options) {
		if (_INSTANCE) {
			return _INSTANCE;
		} else if (!(this instanceof TermsService)) {
			return (_INSTANCE = new TermsService(element, options));
		} else {
			_INSTANCE = this;
		}
		this._init(element, options);
	};
	TermsService.prototype = /** @lends kth.puddingto.TermsService.prototype */ {
		element: null,
		
		options: {
			backButtonSelector: 'A.bk',
			agreementButtonSelector: 'A.ed',
			tabsContainerSelector: '.tmSer UL',
			tabSelector: 'LI',
			contentsText: [
				kth.puddingto.termsOfUseAgreement && kth.puddingto.termsOfUseAgreement.get(i18n.getLocale()),
				kth.puddingto.privacyPolicy && kth.puddingto.privacyPolicy.get(i18n.getLocale())
			],
			contentsUrls: [
				'http://service.pudding.to/v1/user/terms',
				'http://service.pudding.to/v1/user/privacy'
			],
			onBackButtonClick: function() {},
			onAgreementButtonClick: function() {}
		},
		
		_tabsReference: null,
		_tabs: null,
		_contentsContainers: null,
		_iScrolls: null,
		_contentsTexts: null,
		_xhrRequests: null,
		
		_selectedIndex: 0,

		_init: function(element, options) {
			var options = this.options = $.extend(this.options, options);
			
			this._onBackButtonClick = this._onBackButtonClick.bind(this); 
			this._onAgreementButtonClick = this._onAgreementButtonClick.bind(this);
			this._onTabClick = this._onTabClick.bind(this);
			
			this.element = $(element)
					.on('click', this.options.backButtonSelector, this._onBackButtonClick)
					.on('click', this.options.agreementButtonSelector, this._onAgreementButtonClick);
			
			this._tabsReference = 
				$(this.options.tabsContainerSelector).on('click', this.options.tabSelector, this._onTabClick)
					.children(this.options.tabSelector)
						.each(function(index, element) {
							var $content,
								$tab = $(element).data('index', index),
								contentSelector = ((this._tabs || (this._tabs = []))[index] = $tab).children('A').attr('href'),
								contentHtml;
							
							(this._contentsContainers || (this._contentsContainers = []))[index] = $(contentSelector);
							
							if ((contentHtml = this.options.contentsText[index])) {
								this._setContents(index, contentHtml);
							} else {
								this._loadContents(index);						
							}
						}.bind(this));
		},
		
		_loadContents: function(index, onSuccess) {
			if (this._hasContents(index)) {
				return;
			}
			
			PuddingTo.API.showLoader();
			(this._xhrRequests || (this._xhrRequests = []))[index] = $.ajax({
				url: Mustache.to_html(this.options.contentsUrls[index], {
					locale: i18n.getLocale()
				}),
				dataType: 'json',
				success: function(data, status, xhr) {
					var xhrRequestIndex = this._xhrRequests.indexOf(xhr),
						text = data.response;
							
					if (!~xhrRequestIndex || this._hasContents(xhrRequestIndex)) {
						return;
					} else {
						this._setContents(xhrRequestIndex, text);
						if ($.isFunction(onSuccess)) {
							onSuccess.apply(this);
						}	
					}
				}.bind(this),
				error: function(xhr, errorType, error) {
					console.log('load contents error', errorType, error);
				}.bind(this),
				complete: function(xhr, status) {
					console.log('load contents complete', xhr, status);
					PuddingTo.API.hideLoader();
				}.bind(this)
			});
		},
		
		_hasContents: function(index) {
			return !!(this._contentsTexts && this._contentsTexts[index]);
		},
		
		_setContents: function(index, contentHtml) {
			this._contentsContainers[index].children('DIV').html([
				'<pre style="white-space: pre-wrap">',
				((this._contentsTexts || (this._contentsTexts = []))[index] = contentHtml),
				'</pre>'
			].join(''));
			
			if (index === this._selectedIndex) {
				this._render();
			}
		},
		
		_onBackButtonClick: function(event) {
			event.preventDefault();
			this.options.onBackButtonClick.apply(this, [event]);
		},
		
		_onAgreementButtonClick: function(event) {
			event.preventDefault();
			this.options.onAgreementButtonClick.apply(this, [event]);
		},
		
		_onTabClick: function(event) {
			var selectedIndex = Number(event.currentTarget.getAttribute('data-index'));
			event.preventDefault();
			
			if (this._contentsTexts && this._contentsTexts[selectedIndex]) {
				this._selectContent(selectedIndex);	
			} else {
				this._loadContents(selectedIndex, function() {
					this._selectContent(selectedIndex);
				}.bind(this));
			}
		},
				
		_selectContent: function(index) {
			var prevIndex = this._selectedIndex;
			
			if (prevIndex === index) {
				return;
			} else if (!!~prevIndex) {
				this._tabs[prevIndex].removeClass('on');
				this._contentsContainers[prevIndex].removeClass('on');
			}
			
			this._tabs[index].addClass('on');
			this._contentsContainers[index].addClass('on');
			this._selectedIndex = index;
			this._render();
		},
		
		_render: function() {
			var _iScroll;
			if (!(_iScroll = (this._iScrolls || (this._iScrolls = []))[this._selectedIndex])) {
				_iScroll = this._iScrolls[this._selectedIndex] = new iScroll(this._contentsContainers[this._selectedIndex][0], {
					bounce: !$.os.android
				});
				
				setTimeout(function(){
					_iScroll.refresh();	
				}, 1);
			} else {
				_iScroll.scrollTo(0, 0);
			}
		},
		
		hide: function() {
			this.element.hide();
			return this;
		},
		
		show: function() {
			this.element.show();
			return this;
		}
	};
	return TermsService;
})('kth.puddingto.TermsService'));

})(window);